% Created 2012-August-17
% Written by Mihailo R. Jovanovic
% Matlab script that explains how to run dmd_sp_ADMM_complex_polish

% Notation: matrices of snapshots X0Nm1 and X1N
%
%       X0Nm1 = [x_0 ... x_{N-1}]
%       X1N   = [x_1 ... x_N]
%       svd of the matrix of snapshots X0Nm1
%       X0Nm1 = U*S*V'

   clear all
   close all
    
   %...load DMD-preprocessed data 
   load MMM.mat

   %...extract data
   S        = diag(Mih3);
   UstarX1N = Mih1;
   V        = Mih2;

  %...imaginary unit
  ii = sqrt(-1);
  % number of snapshots
  N  = size(S,2);
 
  %...determine optimal DMD matrix using JFM 656 (2010)
  %...Fdmd = U'*X1N*V*inv(S)
  Fdmd = (UstarX1N*V)/S;

  %...rank of Fdmd
  r = rank(Fdmd);      % set the number of states

  % eig-value decomposition of Fdmd
  [Ydmd,Ddmd] = eig(Fdmd);
  Edmd        = diag(Ddmd);      % e-values of the discrete-time system

  %...form Vandermonde matrix
  Vand = zeros(r,N);
  zdmd = Edmd;
  for i = 1:N,
    Vand(:,i) = zdmd.^(i-1);
  end

  %...obtain optimal vector of amplitudes xdmd
  %   objective: minimize the least-squares difference between
  %   the matrix of snapshots X0Nm1 and the linear combination of dmd modes
  %   minimize || Given - L*diag(xdmd)*R ||_F^2

  L     = Ydmd;
  R     = Vand;
  Given = S*V';
  Tmp   = R*Given'*L;

  %...form matrices RRstar and LstarL
  RRstar = R*R';
  LstarL = L'*L;

  %...form matrix P, vector q, and scalar r1 for optimization problem
  %   J = f'*P*f - q'*f - f'*q + r1
  %   f - optimization variable (i.e., the unknown vector of amplitudes)
  P  = (L'*L).*conj(R*R');
  q  = conj(diag(R*Given'*L));
  r1 = trace(Given'*Given);

  %...Cholesky factorization of P
  Pl = chol(P,'lower');

  %...optimal vector of amplitudes xdmd
  xdmd = (Pl')\(Pl\q);

  %...optimal least-squares residual
  Jopt = norm(Given - L*diag(xdmd)*R,'fro')^2;

  %%=====sparsity-promoting algorithm starts here
  %    sparsity-promoting parameter gamma
  %    lower and upper bounds turn out to be relevant for this example
  %    you can adjust the number of grid points if you want to
  gammaval = logspace(1,log10(1600),100);      % tuned to data sequence 

  %...parameter rho (used in the augmented Lagrangian)
  rho = 1;

  % sparse DMD algorithm as a function of gamma
  % Inputs:  matrices {Given,P}
  %          vector q 
  %          gammaval and rho
  %
  % Outputs: xval  ->  vector of amplitudes x as a function of gamma
  %          Jval  ->  J as a function of gamma
  %          Nzval ->  number of nonzero elements of x as a function of gamma
  %          xpol  ->  polished vector of amplitudes x as a function of gamma
  %          Jpol  ->  polished J as a function of gamma
  tic
    [xval,Jval,Nzval,xpol,Jpol] = dmd_sp(Given,P,q,gammaval,rho);
  toc

  %%=====different plots start here
  % polished least-squares residual vs gamma
  figure(1)
  semilogx(gammaval,sqrt(abs(Jpol)),'k+','LineWidth',1,'MarkerSize',7)
  xlab = xlabel('\gamma','interpreter','tex');
  set(xlab,'FontName','cmr10','FontSize',26)
  h = get(gcf,'CurrentAxes');
  set(h,'FontName','cmr10','FontSize',20)
  axis([10 1600 0 180])
  set(gca,'XTick',[10 100 1000])

  % number of non-zero amplitudes vs gamma
  figure(2) 
  semilogx(gammaval,Nzval,'k+','LineWidth',1,'MarkerSize',7)
  xlab = xlabel('\gamma','interpreter','tex');
  set(xlab,'FontName','cmr10','FontSize',26)
  h = get(gcf,'CurrentAxes');
  set(h,'FontName','cmr10','FontSize',20)
  axis([10 1600 0 105])
  set(gca,'XTick',[100 1000 10000])

  % plot spectrum of a DT system for a certain value of gamma
  m = 40;    % Nz = 21
  m = 66;    % Nz = 9
  m = 79;    % Nz = 5
  m = 92;    % Nz = 3

  figure(3)   
  ival = find(xval(:,m));
  plot(real(Edmd),imag(Edmd),'ko',real(Edmd(ival)),imag(Edmd(ival)),'r+', ...
    'LineWidth',1,'MarkerSize',7)
  xlab = xlabel('Re(\mu_i)','interpreter','tex');
  set(xlab,'FontName','cmr10','FontSize',26)
  ylab = ylabel('Im(\mu_i)','interpreter','tex');
  set(ylab,'FontName','cmr10','FontSize',26)
  h = get(gcf,'CurrentAxes');
  set(h,'FontName','cmr10','FontSize',20,'xscale','lin','yscale','lin')
  hold

  %...plot a unit circle
  format compact                    % tightens loose format
  format long e                     % makes numerical output in double precision
  theta = linspace(0,2*pi,100);     % create vector theta
  x = cos(theta);                   % generate x-coordinate
  y = sin(theta);                   % generate y-coordinate
  plot(x,y,'b--','LineWidth',1);    % plot circle
  axis('equal');
  hold

  %...this part shows how to extract certain number of
  %   dominant e-pairs of the DT system

  %...extracting 3 dominant e-values of DT system and
  %   corresponding e-vectors of Fdmd
  m = 92;     % Nz = 3
  ival = find(xval(:,m));
  %...this info is required to visualize dominant DMD modes
  %   3 dominant e-values of DT system identified by sparsity-promoting algorithm
  E1to3 = Edmd(ival);
  %...along with the corresponding e-vectors
  Y1to3 = Ydmd(:,ival);

  %...principal e-pair of Fdmd
  E1 = E1to3(1);
  Y1 = Y1to3(:,1);
