% Created 2012-August-17
% Written by Mihailo R. Jovanovic
%
% Use ADMM to solve the optimization problem
% minimize  J(x) + gamma * || x ||_1
% J(x) = || G - L diag{x} R ||_F^2
% complex-valued data G, L, R and x
%
% J(x) can be equivalently written as
% J(x) = x'*P*x - 2*real(q'*x) + trace(G'*G)
% where
% P = (L'*L).*conjugate(R*R')
% q = conjugate(diag(R*G'*L))
%
% Inputs:  matrices {G,P}
%          vector q
%          gamma and rho
%
% Outputs: xval  ->  vector of amplitudes x as a function of gamma
%          Jval  ->  J as a function of gamma
%          nzval ->  number of nonzero elements of x as a function of gamma
%          xpol  ->  polished vector of amplitudes x as a function of gamma
%          Jpol  ->  polished J as a function of gamma

function [xval,Jval,nzval,xpol,Jpol] = dmd_sp(G,P,q,gamval,rho)

  %...problem data
  n  = length(q);
  I  = eye(n);
  ii = sqrt(-1);

  %...allocate memory for gamma-dependent variables
  Jval  = zeros(1,length(gamval));    % square of Frobenius norm (before polishing)
  Jpol  = zeros(1,length(gamval));    % square of Frobenius norm (after polishing)
  xval  = zeros(n,length(gamval));    % vector of amplitudes (before polishing)
  xpol  = zeros(n,length(gamval));    % vector of amplitudes (after polishing)
  nzval = zeros(1,length(gamval));    % number of non-zero amplitudes

  %...data for the ADMM algorithm
  Max_ADMM_Iter = 10000;
  eps_abs       = 1.e-6;
  eps_rel       = 1.e-4;
  
  %...Cholesky factorization of matrix P + (rho/2)*I
  Prho      = (P + (rho/2)*I);
  Plow      = chol(Prho,'lower');
  Plow_star = Plow';

  for i = 1:length(gamval)
    gamma = gamval(i);

    %...initial conditions
    y = zeros(n,1);
    z = zeros(n,1);

    %...use ADMM to solve the gamma-parameterized problem
    for ADMMstep=1:Max_ADMM_Iter,

      %...x-minimization step
      u = z - (1/rho)*y;
      %...x = (P + (rho)*I)\(q + (rho)*u)
      xnew = Plow_star\(Plow\(q + (rho/2)*u));
      %...xnew = (P + rho*I)\(q + rho*u);

      %...z-minimization step
      a    = (gamma/rho)*ones(n,1);
      v    = xnew + (1/rho)*y;
      znew = ((1 - a./abs(v)).* v).*(abs(v) > a);

      res_dual = rho*norm(znew - z);
      res_prim = norm(xnew - znew);

      %...y-update step
      y = y + rho*(xnew - znew);

      %...stopping criteria
      eps_prim = sqrt(n)*eps_abs + eps_rel*max([norm(xnew),norm(znew)]);
      eps_dual = sqrt(n)*eps_abs + eps_rel*norm(y);

      if (res_prim < eps_prim) && (res_dual < eps_dual)
        break;
      else
        z = znew;
      end

      %...disp([num2str(ADMMstep),'   ',num2str(res_prim,'%6.1E'),'    ',num2str(res_dual,'%6.1E')])
    end

    %...record data
    xval(:,i) = z;                    % vector of amplitudes
    nzval(i)  = nnz(xval(:,i));       % number of non-zero amplitudes
    Jval(i)   = real(z'*P*z) - 2*real(q'*z) + trace(G'*G);     % Frobenius norm (before polishing)

    %...polishing of the nonzero amplitudes
    %   form the constraint matrix E for E^T x = 0
    n = length(z);
    m = n - nnz(z);
    E = zeros(n,m);

    [fsort,find] = sort(abs(z),'descend');
    for k = 1:m,
      E(find(end-m+k),k) = 1;
    end
    E = sparse(E);

    %...form KKT system for the necessary conditions for optimality
    KKT             = [P, E; E', zeros(m,m)];
    right_hand_side = [q; zeros(m,1)];

    %...solve KKT system
    sol = KKT\right_hand_side;

    xpol(:,i) = sol(1:n); % vector of polished amplitudes
    %...polished Frobenius norm
    Jpol(i) = real(xpol(:,i)'*P*xpol(:,i)) - 2*real(q'*xpol(:,i)) + trace(G'*G);

    fprintf('iteration (gamval): %i (out of %i)\n',i,length(gamval))
  end
